@@ -697,10 +697,10 @@ delete_browse (GtkWidget *w, GdkEvent *e, GtkWidget **f)
 {	
 	if (wpFileSelName)
 		g_free (wpFileSelName);
-	wpFileSelName = g_strdup (gtk_file_selection_get_filename
-				  (GTK_FILE_SELECTION (*f)));
-	*f = NULL;
-	return TRUE;
+	wpFileSelName = NULL;
+	
+	fileSel = NULL;
+	return FALSE;
 }
 
 static void
@@ -723,9 +723,13 @@ browse_activated (GtkWidget *w, gchar *s)
 static void
 wp_selection_cancel (GtkWidget *w, GtkWidget **f)
 {
+  gtk_widget_destroy (fileSel);
+  fileSel = NULL;
+
+#if 0
 	GtkWidget *cf = *f;
 	delete_browse (w, NULL, f);
-	gtk_widget_destroy (cf);
+#endif
 }
 
 static void
@@ -794,20 +798,108 @@ wp_selection_ok (GtkWidget *w, GtkWidget **f)
 {
 	GtkWidget *cf = *f;
 	
-	if (w)
-		delete_browse (w, NULL, f);
-	/* printf ("wp ok\n"); */
+	if (w) {
+	  if (wpFileSelName)
+		g_free (wpFileSelName);
+	  wpFileSelName = g_strdup (gtk_file_selection_get_filename
+				  (GTK_FILE_SELECTION (*f)));
 
-	gtk_widget_destroy (cf);
-	set_monitor_filename (wpFileSelName);
+	  if (!g_file_test(wpFileSelName, G_FILE_TEST_ISFILE)) {
+	    g_free(wpFileSelName);
+	    wpFileSelName = NULL;
+	  }
+
+	}
+
+	gtk_widget_destroy(fileSel);
+        fileSel = NULL;
+
+	if (wpFileSelName)
+	    set_monitor_filename (wpFileSelName);
+
+}
+static void
+setup_preview(GtkWidget *widget)
+{
+	char *p;
+	GList *l;
+	GtkWidget *pp = NULL;
+	GdkImlibImage *im;
+	int w,h;
+	GtkWidget *frame;
+	GtkFileSelection *fs;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	frame = gtk_object_get_data(GTK_OBJECT(widget),"frame");
+	fs = gtk_object_get_data(GTK_OBJECT(frame),"fs");
+
+	if((l = gtk_container_children(GTK_CONTAINER(frame))) != NULL) {
+		pp = l->data;
+		g_list_free(l);
+	}
+
+	if(pp)
+		gtk_widget_destroy(pp);
+	
+	p = gtk_file_selection_get_filename(fs);
+	if(!p || !g_file_test (p,G_FILE_TEST_ISLINK|G_FILE_TEST_ISFILE) ||
+	   !(im = gdk_imlib_load_image (p)))
+		return;
+
+	w = im->rgb_width;
+	h = im->rgb_height;
+	if(w>h) {
+		if(w>100) {
+			h = h*(100.0/w);
+			w = 100;
+		}
+	} else {
+		if(h>100) {
+			w = w*(100.0/h);
+			h = 100;
+		}
+	}
+	pp = gnome_pixmap_new_from_imlib_at_size (im, w, h);
+	gtk_widget_show(pp);
+	gtk_container_add(GTK_CONTAINER(frame),pp);
+
+	gdk_imlib_destroy_image(im);
 }
 
 static void
 browse_wallpapers (GtkWidget *w, gpointer p)
 {
-    if (!fileSel) {
-	
-	fileSel = gtk_file_selection_new (_("Wallpaper Selection"));
+  GtkWidget *hbox, *widg;
+  if (!fileSel) {
+    
+    fileSel = gtk_file_selection_new (_("Wallpaper Selection"));
+    hbox = GTK_FILE_SELECTION (fileSel)->file_list;
+    do {
+      hbox = hbox->parent;
+      if(!hbox) {
+	g_warning(_("Can't find an hbox, using a normal file "
+		    "selection"));
+	return;
+      }
+    } while(!GTK_IS_HBOX(hbox));
+	widg = gtk_frame_new(_("Preview"));
+	gtk_widget_show(widg);
+	gtk_box_pack_end(GTK_BOX(hbox),widg,FALSE,FALSE,0);
+	gtk_widget_set_usize(widg,110,110);
+	gtk_object_set_data(GTK_OBJECT(widg),"fs",fileSel);
+	gtk_object_set_data(GTK_OBJECT(GTK_FILE_SELECTION (fileSel)->file_list),
+			    "frame",widg);
+	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION (fileSel)->file_list),"select_row",
+			   GTK_SIGNAL_FUNC(setup_preview),NULL);
+	gtk_object_set_data(GTK_OBJECT(GTK_FILE_SELECTION (fileSel)->selection_entry),"frame",widg);
+	gtk_signal_connect_while_alive(GTK_OBJECT(GTK_FILE_SELECTION (fileSel)->selection_entry),
+				       "changed",
+				       GTK_SIGNAL_FUNC(setup_preview),NULL,
+				       GTK_OBJECT(fileSel));
+
+
 	if (wpFileSelName)
 	    gtk_file_selection_set_filename (GTK_FILE_SELECTION (fileSel),
 					     wpFileSelName);
