@@ -21,7 +21,7 @@ struct _EditorDescription
         gboolean accepts_lineno;
 };
 
-EditorDescription original_info;
+EditorDescription original_info = { NULL };
 
 EditorDescription possible_editors[] =
 {
@@ -102,7 +102,9 @@ edit_read(void)
                 = gnome_config_get_string("/editor/Editor/EDITOR=emacs");
         original_info.executable_name = original_name;
         set_combo( original_name );
-        original_info.name = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry));
+        if(original_info.name)
+                g_free(original_info.name);
+        original_info.name = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry)));
         original_info.needs_term
                 = gnome_config_get_bool("/editor/Editor/NEEDS_TERM=TRUE");
         original_info.execution_type
@@ -111,6 +113,23 @@ edit_read(void)
                 = gnome_config_get_bool("/editor/Editor/ACCEPTS_LINE_NO=TRUE");
 }
 
+static void
+edit_changed (GtkWidget *widget, gpointer data)
+{
+        capplet_widget_state_changed(CAPPLET_WIDGET (capplet), TRUE);
+}
+
+static void
+combo_changed (GtkWidget *widget, gpointer data)
+{
+        EditorDescription desc = get_combo ();
+        if ( checkbox != NULL )
+                gtk_toggle_button_set_active
+                        ( GTK_TOGGLE_BUTTON( checkbox ),
+                          desc.needs_term );
+        edit_changed( widget, data );
+}
+
 static void
 edit_help(void)
 {
@@ -153,27 +172,11 @@ edit_revert (void)
         gnome_config_set_string("/editor/Editor/EDITOR_TYPE", original_info.execution_type );
         gnome_config_set_bool("/editor/Editor/ACCEPTS_LINE_NO", original_info.accepts_lineno);
         gnome_config_sync();
+
         edit_read ();
         edit_sync (original_info);
 }
 
-static void
-edit_changed (GtkWidget *widget, gpointer data)
-{
-        capplet_widget_state_changed(CAPPLET_WIDGET (capplet), TRUE);
-}
-
-static void
-entry_changed (GtkWidget *widget, gpointer data)
-{
-        EditorDescription desc = get_combo ();
-        if ( checkbox != NULL )
-                gtk_toggle_button_set_active
-                        ( GTK_TOGGLE_BUTTON( checkbox ),
-                          desc.needs_term );
-        edit_changed( widget, data );
-}
-
 static void
 edit_setup(void)
 {
@@ -217,8 +220,10 @@ edit_setup(void)
         gtk_signal_connect (GTK_OBJECT (capplet), "cancel",
                             GTK_SIGNAL_FUNC (edit_revert), NULL);
 
-        gtk_signal_connect (GTK_OBJECT (GTK_COMBO(combo)->entry), "changed",
-                            GTK_SIGNAL_FUNC (entry_changed), NULL);
+        gtk_signal_connect (GTK_OBJECT (GTK_COMBO(combo)->entry), "activate",
+                            GTK_SIGNAL_FUNC (combo_changed), NULL);
+        gtk_signal_connect (GTK_OBJECT (GTK_COMBO(combo)->list), "selection_changed",
+                            GTK_SIGNAL_FUNC (combo_changed), NULL);
 
         gtk_signal_connect (GTK_OBJECT (checkbox), "toggled",
                             GTK_SIGNAL_FUNC (edit_changed), NULL);
@@ -245,5 +250,8 @@ main (int argc, char **argv)
         edit_setup ();
         capplet_gtk_main ();
 
+        if(original_info.name)
+                g_free(original_info.name);
+
 	return 0;
 }
