@@ -7,11 +7,11 @@
 #include "gnome.h"
 #include <regex.h>
 /* Typedefs */
-void init_mime_info ();
-void discard_mime_info ();
+void init_mime_info (void);
+void discard_mime_info (void);
 void set_mime_key_value (gchar *mime_type, gchar *key, gchar *value);
 const char * local_mime_get_value (const char *mime_type, char *key);
-void write_user_keys ();
-void write_initial_keys ();
+void write_user_keys (void);
+void write_initial_keys (void);
 
 #endif
