@@ -214,6 +214,7 @@ create_list (GtkCList *list, gchar *directory)
 
         for (listp = desktop_items; listp; listp = g_list_next(listp)) {
                 gchar *name;
+                gchar *full_name;
 
                 name = listp->data;
                 prefix = g_strconcat ("=", directory, name, "=/Desktop Entry/", NULL);
@@ -222,9 +223,26 @@ create_list (GtkCList *list, gchar *directory)
 
                 /* Lets make the data for the screensaver */
                 sdnew = g_malloc (sizeof (screensaver_data));
+                sdnew->tryexec = gnome_config_get_string ("TryExec");
+                /* do we really want to load this? */
+#ifdef HAVE_REDHAT_XSCREENSAVER_RPM
+                if (sdnew->tryexec == NULL) {
+                        g_free (sdnew);
+                        gnome_config_pop_prefix ();
+                        continue;
+                }
+                full_name = g_strconcat ("/usr/X11R6/lib/xscreensaver/", sdnew->tryexec, NULL);
+                if (!g_file_test (full_name, G_FILE_TEST_ISFILE)) {
+                        g_free (full_name);
+                        g_free (sdnew->tryexec);
+                        g_free (sdnew);
+                        gnome_config_pop_prefix ();
+                        continue;
+                }
+                g_free (full_name);
+#endif
                 sdnew->desktop_filename = g_strconcat (directory, name, NULL);
                 sdnew->name = gnome_config_get_translated_string ("Name");
-                sdnew->tryexec = gnome_config_get_string ("TryExec");
                 gnome_config_pop_prefix ();
                 sdnew->args = NULL;
                 
@@ -233,7 +251,7 @@ create_list (GtkCList *list, gchar *directory)
                 g_free (prefix);
                 sdnew->windowid = gnome_config_get_string ("WindowIdCommand");
                 sdnew->root = gnome_config_get_string ("RootCommand");
-                sdnew->author = gnome_config_get_string ("Author");
+                sdnew->author = gnome_config_get_translated_string ("Author");
                 sdnew->comment = gnome_config_get_translated_string ("ExtendedComment");
                 sdnew->demo = gnome_config_get_string ("Demo");
                 prefix =  gnome_config_get_string ("Icon");
