@@ -14,6 +14,10 @@
 #include "capplet-widget.h"
 #include <gnome.h>
 
+/* use this flag so we don't tell control center of changes that it */
+/* shouldn't turn on Try/Revert because we changed it, not the user */
+static gboolean wecare=FALSE;
+static GtkWidget *dialog;
 typedef struct {
     GtkWidget *capplet;
 
@@ -64,6 +68,8 @@ static void ui_do_revert(GtkWidget *w, SoundProps *props);
 static void ui_do_ok(GtkWidget *w, SoundProps *props);
 static void ui_do_help(void);
 static void ui_do_cancel(GtkWidget *w, SoundProps *props);
+static void ui_do_show(GtkWidget *w, SoundProps *props);
+static void ui_do_hide(GtkWidget *w, SoundProps *props);
 static void sound_properties_play_sound(GtkWidget *btn, SoundProps *props); 
 static void reload_esd_samples(const char *config_path);
 static void reload_all_esd_samples(void);
@@ -189,6 +195,9 @@ sound_properties_create(void)
     gboolean btmp;
     SoundProps *retval;
 
+    /* dont tell control center about changes */
+    wecare=FALSE;
+
     retval = g_new0(SoundProps, 1);
 
     retval->by_category = g_hash_table_new(g_str_hash, g_str_equal);
@@ -211,6 +220,10 @@ sound_properties_create(void)
                        ui_do_ok, retval);
     gtk_signal_connect(GTK_OBJECT(retval->capplet), "cancel",
                        ui_do_cancel, retval);
+    gtk_signal_connect(GTK_OBJECT(retval->capplet), "page_hidden",
+                       ui_do_hide, retval);
+    gtk_signal_connect(GTK_OBJECT(retval->capplet), "page_shown",
+                       ui_do_show, retval);
 
     retval->notebook = notebook = gtk_notebook_new();
 
@@ -297,6 +310,10 @@ sound_properties_create(void)
 
     sound_properties_regenerate_ctree(retval);
 
+/*    gtk_clist_set_column_auto_resize(GTK_CLIST(retval->ctree), 0, TRUE); */
+    gtk_clist_set_column_auto_resize(GTK_CLIST(retval->ctree), 1, TRUE);
+    gtk_clist_set_column_auto_resize(GTK_CLIST(retval->ctree), 2, TRUE);
+
     gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
     wtmp = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_set_usize(GTK_WIDGET(wtmp),
@@ -325,6 +342,8 @@ sound_properties_create(void)
 
     gtk_signal_connect(GTK_OBJECT(retval->ctree), "tree_select_row",
                        sound_properties_event_select, retval);
+    gtk_clist_select_row(GTK_CLIST(retval->ctree), 1, 0);
+
     gtk_signal_connect(GTK_OBJECT(retval->btn_play), "clicked",
                        sound_properties_play_sound, retval);
     gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(retval->btn_filename))),
@@ -338,6 +357,8 @@ sound_properties_create(void)
 
     gtk_widget_show_all(retval->capplet);
 
+    /* ok we might care now */
+    wecare = TRUE;
     return retval;
 }
 
@@ -355,11 +376,18 @@ static void
 sound_properties_regenerate_ctree(SoundProps *props)
 {
     GString *tmpstr = g_string_new(NULL);
+    GtkWidget *entry;
     char *ctmp, *ctmp2;
+    GtkWidget *node;
 
     gtk_clist_freeze(GTK_CLIST(props->ctree));
     gtk_clist_clear(GTK_CLIST(props->ctree));
 
+    /* clean up hash tables - key is first arg to GHFunc */
+    g_hash_table_foreach(props->by_category, (GHFunc) g_free, NULL);
+    g_hash_table_destroy(props->by_category);
+    props->by_category = g_hash_table_new(g_str_hash, g_str_equal);
+
     ctmp = gnome_config_file("/sound/events");
     if(ctmp)
         sound_properties_read_path(props, tmpstr, ctmp);
@@ -371,6 +399,7 @@ sound_properties_regenerate_ctree(SoundProps *props)
     g_free(ctmp);
 
     g_string_free(tmpstr, TRUE);
+
     gtk_clist_thaw(GTK_CLIST(props->ctree));
 }
 
@@ -399,6 +428,8 @@ sound_properties_read_path(SoundProps *props,
     struct dirent *dent;
     GHashTable *cathash;
 
+    char *file;
+
     dirh = opendir(path);
     if(!dirh)
         return;
@@ -408,25 +439,26 @@ sound_properties_read_path(SoundProps *props,
             || !strcmp(dent->d_name, ".."))
 		    continue;
 
-        g_string_sprintf(tmpstr, "=%s/%s=", path, dent->d_name);
+        file = g_strdup(dent->d_name);
+
+        g_string_sprintf(tmpstr, "=%s/%s=", path, file);
 
         gnome_config_push_prefix(tmpstr->str);
 
-        arow[1] = NULL; arow[2] = NULL;
+        arow[1] = ""; arow[2] = "";
 
         ctmp = gnome_config_get_translated_string("__section_info__/description");
         if(ctmp && *ctmp) {
             arow[0] = ctmp;
         } else {
             g_free(ctmp);
-            arow[0] = g_strdup(dent->d_name);
+            arow[0] = g_strdup(file);
             if(strstr(arow[0], ".soundlist")) {
                 *strstr(arow[0], ".soundlist") = '\0';
             }
         }
 
-        category_node = g_hash_table_lookup(props->by_category,
-                                            dent->d_name);
+        category_node = g_hash_table_lookup(props->by_category, file);
 
         if(!category_node) {
             category_node = gtk_ctree_insert_node(GTK_CTREE(props->ctree),
@@ -436,11 +468,12 @@ sound_properties_read_path(SoundProps *props,
                                                   FALSE);
             gtk_ctree_node_set_selectable(GTK_CTREE(props->ctree), category_node,
                                           FALSE);
-            gtk_ctree_node_set_row_data(GTK_CTREE(props->ctree),
+            gtk_ctree_node_set_row_data_full(GTK_CTREE(props->ctree),
                                         category_node,
                                         g_hash_table_new(g_str_hash,
-                                                         g_str_equal));
-            g_hash_table_insert(props->by_category, g_strdup(dent->d_name),
+                                                         g_str_equal),
+                                        (GtkDestroyNotify)g_hash_table_destroy);
+            g_hash_table_insert(props->by_category, g_strdup(file),
                                 category_node);
         }
 
@@ -484,7 +517,7 @@ sound_properties_read_path(SoundProps *props,
 
             } else {
 
-                arow[0] = NULL;
+                arow[0] = "";
                 g_string_sprintf(tmpstr, "%s/description", sample_name);
                 arow[1] = gnome_config_get_translated_string(tmpstr->str);
                 if(!arow[1] || !*arow[1]) {
@@ -499,7 +532,7 @@ sound_properties_read_path(SoundProps *props,
                                                    FALSE);
                 g_free(arow[1]);
                 new_entry = g_new0(SoundEvent, 1);
-                new_entry->category = g_strdup(dent->d_name);
+                new_entry->category = g_strdup(file);
                 new_entry->name = sample_name;
                 new_entry->file = sample_file;
                 new_entry->row = event_node;
@@ -517,9 +550,11 @@ sound_properties_read_path(SoundProps *props,
 
                 prev_event_node = event_node;
             }
+            gtk_ctree_expand(GTK_CTREE(props->ctree), GTK_CTREE_NODE(category_node));
         }
 
         gnome_config_pop_prefix();
+        g_free(file);
     }
     closedir(dirh);
 }
@@ -537,17 +572,27 @@ sound_properties_event_play(GtkWidget *widget, SoundProps *props)
 static void
 sound_properties_event_change_file(GtkEditable *entry, SoundProps *props)
 {
+    char *tmp, *sounddir, *tmp2;
+    
     if(props->ignore_changed)
         return;
 
-    g_return_if_fail(GTK_CLIST(props->ctree)->selection);
+/*    g_return_if_fail(GTK_CLIST(props->ctree)->selection); */
+    if (!GTK_CLIST(props->ctree)->selection)
+        return;
 
     capplet_widget_state_changed(CAPPLET_WIDGET(props->capplet), TRUE);
 
+    tmp = gtk_entry_get_text(GTK_ENTRY(entry));
+    sounddir = gnome_sound_file("");
+    if (!strncmp(tmp, sounddir, strlen(sounddir)))
+        tmp2 = tmp+strlen(sounddir);
+    else
+        tmp2 = tmp;
+
     gtk_ctree_node_set_text(GTK_CTREE(props->ctree),
                             GTK_CLIST(props->ctree)->selection->data,
-                            2,
-                            gtk_entry_get_text(GTK_ENTRY(entry)));
+                            2, tmp2);
 }
 
 static void
@@ -586,7 +631,7 @@ sound_properties_event_apply(GtkCTree *ctree,
     gtk_ctree_node_get_text(GTK_CTREE(ctree),
                             node, 2, &cur_filename);
 
-    if(!cur_filename) return;
+    if(!cur_filename || !(*cur_filename)) return;
 
     /* If the user didn't change the setting, no need to set it */
     if(!strcmp(cur_filename, ev->file))
@@ -628,13 +673,15 @@ sound_properties_set_sensitivity(GtkToggleButton *btn,
 
     gtk_widget_set_sensitive(gtk_notebook_get_tab_label(GTK_NOTEBOOK(props->notebook),
                                                         page), do_events);
-
-    capplet_widget_state_changed(CAPPLET_WIDGET(props->capplet), TRUE);
+    if (wecare)
+        capplet_widget_state_changed(CAPPLET_WIDGET(props->capplet), TRUE);
 }
 
 static void
 ui_do_revert(GtkWidget *w, SoundProps *props)
 {
+    wecare=FALSE;
+
     gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(props->enable_esd_startup),
                                 gnome_config_get_bool("/sound/system/settings/start_esd=true"));
 
@@ -642,6 +689,10 @@ ui_do_revert(GtkWidget *w, SoundProps *props)
                                 gnome_config_get_bool("/sound/system/settings/event_sounds=true"));
 
     sound_properties_regenerate_ctree(props);
+
+    gtk_clist_select_row(GTK_CLIST(props->ctree), 1, 0);
+
+    wecare=TRUE;
 }
 
 static void
@@ -668,6 +719,18 @@ ui_do_cancel(GtkWidget *w, SoundProps *props)
 {
     gtk_main_quit();
 }
+static void
+ui_do_show(GtkWidget *w, SoundProps *props)
+{
+    if (dialog)
+        gtk_widget_show (dialog);
+}
+static void
+ui_do_hide(GtkWidget *w, SoundProps *props)
+{
+    if (dialog)
+        gtk_widget_hide (dialog);
+}
 
 static void
 sound_properties_play_sound(GtkWidget *btn, SoundProps *props)
@@ -688,18 +751,21 @@ sound_properties_play_sound(GtkWidget *btn, SoundProps *props)
     if(ctmp2 && g_file_exists(ctmp2))
        gnome_sound_play(ctmp2);
     else {
-        GnomeDialog *dlg;
         char *msg;
 
         if(ctmp2 && ctmp2[0]=='/')
             msg = _("The sound file for this event does not exist.");
         else
-            msg = _("The sound file for this event does not exist. You may want to install the gnome-audio package for a set of default sounds.");
+            msg = _("The sound file for this event does not exist.\n"
+                    "You may want to install the gnome-audio package\n"
+                    "for a set of default sounds.");
 
-        dlg = GNOME_DIALOG(gnome_message_box_new(msg,
-                                                 GNOME_MESSAGE_BOX_ERROR,
-                                                 _("Close"), NULL));
-        gnome_dialog_run(dlg);
+        dialog = gnome_message_box_new(msg,
+                                       GNOME_MESSAGE_BOX_ERROR,
+                                       _("Close"), NULL);
+        gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
+        gnome_dialog_run(GNOME_DIALOG (dialog));
+        dialog = NULL;
     }
 
     g_free(ctmp2);
