@@ -215,7 +215,7 @@ launch_capplet (node_data *data, gboolean exec_new)
         }
         if (data->notetab_id != -1) {
                 gtk_notebook_set_page (GTK_NOTEBOOK (notebook), data->notetab_id);
-                if (data->capplet) {
+                if (data->capplet && exec_new) {
                         GNOME_capplet_page_shown (data->capplet,
                                                   data->id,
                                                   &ev);
