@@ -13,7 +13,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
-void launch_capplet (node_data *data);
+void launch_capplet (node_data *data, gboolean exec_new);
 node_data *find_node_by_id (gint id);
 #ifdef __cplusplus
 }
