@@ -186,7 +186,9 @@ capplet_widget_multi_new (gint capid)
 void
 capplet_widget_state_changed(CappletWidget *cap, gboolean undoable)
 {
+        g_print ("in capplet_widget_state_changed\n");
         if (cap->changed == FALSE) {
+                g_print ("\tchanging state\n");
                 capplet_corba_state_changed (cap->control_center_id, undoable);
                 cap->changed = TRUE;
         }
