@@ -11,6 +11,7 @@
 
 #ifdef __cplusplus
 extern "C" {
+#pragma }
 #endif /* __cplusplus */
 
 
@@ -21,6 +22,13 @@ gint button_press (GtkCTree *ctree, GdkEventButton *event, gpointer data);
 
 /* callbacks */
 void selected_row_callback (GtkWidget *widget, GtkCTreeNode *node, gint column);
+        
+typedef enum
+{
+        CAPPLET_INACTIVE,
+        CAPPLET_UNREGISTERED,
+        CAPPLET_ACTIVE
+} CappletState;
 
 typedef struct _node_data node_data;
 struct _node_data
@@ -39,11 +47,11 @@ struct _node_data
         GtkWidget *ok_button;
         GtkWidget *cancel_button;
         GtkWidget *help_button;
+        CappletState state;
 };
 
 
 
-
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
